library pr_geo;
//Puerto Rico Geographic measurement system
import 'package:flutter/cupertino.dart';
import 'dart:math';

//TODO: Add an option to input distance in different units
class GeoCoordinate{
  final double latitude;
  final double longitude;
  //Height over the sea level
  final double altitude;
  GeoCoordinate({@required this.latitude,@required this.longitude,this.altitude = 0});
}
// ignore: camel_case_types
class PR_Geo{
  //Mean sea level in meters
  static const double SeaLevel = 6371146;
  static double _degreesToCoordinate({@required double angle,@required double altitude}){
    //Calculate the radius of the coordinate
    double r = altitude + SeaLevel;
    //Calculate the Circumference
    double C = 2 * pi * r;
    double position;
    if(0 <= angle && angle <= 180){
      position = angle/360 * C;
    }else if(-180 <= angle && angle < 0){
      position = -(angle.abs()/360 * C);
    }else if(180 < angle && angle <= 360){
      position = -1 * ((angle - 180)/360 * C);
    }else if(-360 <= angle && angle < -180){
      position = (angle.abs() - 180)/360 * C;
    }else{
      //Accept negative coordinates in the future
      throw 'Coordinates out of range: angle must be between -360 and 360 degrees';
    }
    return position;
  }
  //Distance is given in kilometers
  static double distance(GeoCoordinate coordinate1,GeoCoordinate coordinate2){
    //Perform vertical cross section calculations
    double lonX1 = _degreesToCoordinate(angle: coordinate1.longitude, altitude: coordinate1.altitude);
    double lonX2 = _degreesToCoordinate(angle: coordinate2.longitude, altitude: coordinate2.altitude);
    double deltaX1 = lonX1 - lonX2;
    //Perform horizontal cross section calculations
    double latX1 = _degreesToCoordinate(angle: coordinate1.latitude, altitude: coordinate1.altitude);
    double latX2 = _degreesToCoordinate(angle: coordinate2.latitude, altitude: coordinate2.altitude);
    double deltaX2 = latX1 - latX2;
    //Use a top view where the planes get merged and now I can use them as if it where a cartesian plane
    double totalDistance = pow(deltaX1, 2) + pow(deltaX2,2) + pow(coordinate1.altitude - coordinate2.altitude, 2);
    totalDistance = sqrt(totalDistance);
    return totalDistance;
  }
}